/*
 *	MAX10SoM-I2C.c
 *
 *	Created on: 2015/04/30
 *		Author: jinso(dung)
 */

/* Include file */
#include "../inc/driver-common.h"
#include "../inc/MAX10SoM-I2C.h"

/****************************************************************
void I2C_init
	This function initialize the prescalor for the SCL
	and then enables the I2C. This must be run before
	any other i2c code is executed
inputs
	board_info:information for i2c,example:
		the base address of the component,
		speed for communication(speed is 100K, 400K ... (in Hz))
		frequency of the clock driving this component ( in Hz)
return value
	void
*****************************************************************/
void I2C_init(struct I2C_info *board_info)
{
	alt_u32 prescale = (board_info->clk/( 5 * board_info->speed))-1;
	alt_u32 base_add = board_info->address;
#ifdef  DRI_DEBUG
	printf(" Initializing  I2C at 0x%x, \n\tClock source 0x%x \n\t Speed 0x%x \n\tPrescale 0x%x\n",base_add,board_info->clk,board_info->speed,prescale);
#endif

	IOWR(base_add, I2C_CTR, 0x00);
	/* Clean any pending IRQ*/
	IOWR(base_add, I2C_CR, CR_IACK);

	/* Load low presacle bit*/
	IOWR(base_add, I2C_PRERL, 0xff & prescale);

	/* Load upper prescale bit */
	IOWR(base_add, I2C_PRERH, 0xff & (prescale>>8));

	/* Turn on the core*/
	IOWR(base_add, I2C_CTR,CTR_EN);
}

/****************************************************************
alt_u8 I2C_start
	Sets the start bit and then sends the first byte which
	is the address of the device + the write bit.
inputs
	board_info :Informations of the component
	device_name:Name of the device
	dir        :1 = read, 0 = write
return value
	0 if address is acknowledged
	1 if address was not acknowledged
*****************************************************************/
alt_u8 I2C_start(struct I2C_info *board_info, char *device_name, alt_u8 dir)
{
	alt_u32 base_add = board_info->address;
	struct I2C_dev_info dev;
	alt_u8 i;

	for( i = 0; i < ARRAY_SIZE(board_info->dev_info); i++ ) {
		if (strcmp(device_name,board_info->dev_info[i].device_name) == 0) {
			dev = board_info->dev_info[i];
		}
	}

#ifdef  DRI_DEBUG
	printf(" Start  I2C at 0x%x, \n\twith address 0x%x \n\tand direction 0x%x\n",base_add,dev.slave_addr,dir);
#endif
	/* Transmit the address shifted by one and add the read/write bit*/
	IOWR(base_add, I2C_TXR, (( dev.slave_addr <<1 ) + ( 0x01 & dir )));

	/* Set start and write  bits which will start the transaction*/
	IOWR(base_add, I2C_CR, CR_STA | CR_WR);

	/* Wait for the transaction to be over */
	while( IORD(base_add, I2C_SR) & SR_TIP );

	/* Now check to see if the address was acknowledged */
	if( IORD(base_add, I2C_SR) & SR_RxACk ){
#ifdef DRI_DEBUG
		printf("\tNOACK\n");
#endif
		return I2C_NACK;
	}
	else{
#ifdef DRI_DEBUG
		printf("\tACK\n");
#endif
		return I2C_ACK;
	}
}

/****************************************************************
alt_u8 I2C_read
	assumes that any addressing and start
	has already been done.reads one byte of data from the slave.
	on the last read, we don't acknowledge and set the stop bit
inputs
	board_info:Informations of the component
	last      :if is the last read there must not be a ack
return value
	byte read back
*****************************************************************/
alt_u8 I2C_read(struct I2C_info *board_info, alt_u8 last)
{
	alt_u32 base_add = board_info->address;

#ifdef DRI_DEBUG
	printf(" Read I2C at 0x%x, \n\twith last0x%x\n",base_add,last);
#endif
	/* Setting for read data */
	if(last){
		/* Start a read with noack and stop bit*/
		IOWR(base_add, I2C_CR, CR_RD | CR_ACK | CR_STO);
	}else{
		/* Start read*/
		IOWR(base_add, I2C_CR, CR_RD);
	}

	/* Wait for the transaction to be over */
	while(IORD(base_add, I2C_SR) & SR_TIP);

	/* Now read the data */
	return (IORD(base_add, I2C_RXR));
}

/****************************************************************
alt_u8 I2C_write
	assumes that any addressing and start has already been done.
	writes one byte of data from the slave.
	If last is set the stop bit set.
inputs
	board_info:Informations of the component
	data      :data byte to write
	last      :on the last write there must not be a ack
return value
	0 if address is acknowledged
	1 if address was not acknowledged
*****************************************************************/
alt_u8 I2C_write(struct I2C_info *board_info, alt_u8 data, alt_u8 last)
{
	alt_u32 base_add = board_info->address;

#ifdef  DRI_DEBUG
	printf(" Read I2C at 0x%x, \n\twith data 0x%x,\n\twith last0x%x\n",base_add,data,last);
#endif
	/* Transmit the data */
	IOWR(base_add, I2C_TXR, data);

	/* Setting for write data */
	if(last){
		/* Start a write with noack and stop bit*/
		IOWR(base_add, I2C_CR, CR_WR | CR_STO);
	}else{
		/* Start write */
		IOWR(base_add, I2C_CR, CR_WR);
	}

	/* Wait for the transaction to be over */
	while(IORD(base_add, I2C_SR) & SR_TIP);

	/* Now check to see if the address was acknowledged */
	if(IORD(base_add, I2C_SR) & SR_RxACk){
#ifdef DRI_DEBUG
		printf("\tNOACK\n");
#endif
		return I2C_NACK;
	}else{
#ifdef DRI_DEBUG
		printf("\tACK\n");
#endif
		return I2C_ACK;
	}
}
