/*
 * MAX10SoM-SPI.c
 *
 *  Created on: 2015/05/18
 *      Author: Hiep
 */

/* Include file */
#include "../inc/driver-common.h"
#include "../inc/MAX10SoM-SPI.h"
#include "../inc/MAX10SoM-PIO.h"
#include "../../API-CORE/inc/PIO-core.h"

/****************************************************************
void SPI_init()
	This function initialize SPI interface for 4 channels
	It must be run first before any other spi function is executed
inputs
	board_info:the information of:
		address : Base address for each channel(SPI0, SPI1, SPI2, SPI3's base address),
		spi_mode : spi transfer mode for each channel(SPI_MODE0, SPI_MODE1, SPI_MODE2, SPI_MODE3),
		speed : Clock for each channel communication,
		clk: System Clock source.
return value
	no
*****************************************************************/
void SPI_init(struct SPI_info *board_info)
{
	alt_u8 enable, BR;
	alt_u32 CLK = board_info->clk;
	alt_u32 SPI_base = board_info->address;
	alt_u8 mode = board_info->spi_mode;
#ifdef  DRI_DEBUG
	printf(" Initializing  SPI at 0x%x, \n\t Clock source 0x%x, /"
			"\n\t Speed of each SPIx 0x%x ,\n\t Transfer mode 0x%x\n",SPI_base, CLK, board_info->speed, mode);
#endif
	IOWR(SPI_base, SPI_CONTROL_REG, 0x00);				/*SPI Disable*/
	BR = CLK/((board_info->speed) * 2)-1;
	IOWR(SPI_base, SPI_BR_REG, BR);
	enable = SPI_CONTROL_INTE;
	IOWR(SPI_base, SPI_CONTROL_REG, mode | enable);		/*SPI START*/
}

/****************************************************************
alt_u8 SPI_write()
	Writing the write_length bytes of data then send to device.
inputs
	board_info  :The informations of:
		address :Base address for each channel(SPI0, SPI1, SPI2, SPI3's base address),
	write_data  :Pointer which points to the store send-data to device
	write_length: n Bytes of transmission data (n = 0,1,2,...).
return value
	0:if writing data successfully.
*****************************************************************/
alt_u8 SPI_write(struct SPI_info *board_info, const alt_u8 * write_data, alt_u16 write_length)
{
	alt_u32 SPI_base = board_info->address;
	alt_u16 remain = write_length;
	/* Before write a data make dummy read */
	IORD(SPI_base, SPI_RXDATA_REG);
	while (remain > 0)
	{
		/* Waiting until spi is ready for transmission then Writing data */
		spi_ready_wait(SPI_base);
		IOWR(SPI_base, SPI_TXDATA_REG, *write_data++);
		/* Reading Dummy*/
		spi_ready_wait(SPI_base);
		IORD(SPI_base, SPI_RXDATA_REG);
		remain--;
	}
	return 0;
}

/****************************************************************
alt_u8 SPI_read()
	Reading the read_length bytes of data from device.
inputs
	board_info  :The informations of:
		address :Base address for each channel(SPI0, SPI1, SPI2, SPI3's base address),
	read_data   :Pointer which points to the store read-data from device
	read_length :n Bytes of reception data (n = 0,1,2,...).
return value
	0:if reading data successfully.
*****************************************************************/
alt_u8 SPI_read(struct SPI_info *board_info, alt_u8 * read_data, alt_u16 read_length )
{
	alt_u8 status;
	alt_u32 SPI_base = board_info->address;
	alt_u16 remain = read_length;
	/* Before read a data make dummy read */
	IORD(SPI_base, SPI_RXDATA_REG);
	while (remain > 0) {
		/* Writing Dummy to generate clock */
		spi_ready_wait(SPI_base);
		IOWR(SPI_base, SPI_TXDATA_REG, *read_data);
		/* Reading Data*/
		spi_ready_wait(SPI_base);
		alt_u32 rxdata = IORD(SPI_base, SPI_RXDATA_REG);
		*read_data++ = (alt_u8)rxdata;

		remain--;
	}
	/* Wait until the interface has finished transmitting */
	do
	{
		status = IORD(SPI_base, SPI_STATUS_REG);
	} while ((status & SPI_STATUS_TXEMP) == 0);

	return 0;
}

/*********************************************************************************
void spi_ready_wait()
	Checking Status register until data sending/receiving status is ready.
inputs
	Addr: Base address for each channel(SPI0, SPI1, SPI2, SPI3's base address),
return value
	no
**********************************************************************************/
void spi_ready_wait(alt_u32 addr)
{
	alt_u32 SPI_base = addr;
	while(((SPI_STATUS_TXRDY | SPI_STATUS_TXEMP) & (IORD(SPI_base, SPI_STATUS_REG))) != 0x03 );
}

/*********************************************************************************
void spi_clear_cs()
	Set chip select pin HIGH to stop transfer.
inputs
	pio_cs: Chip select pin Number,
return value
	no
**********************************************************************************/
void SPI_clear_cs( struct SPI_info *board_info)
{
	alt_u8 PIO_num = board_info->dev_info->pio_cs;
	pio_direction(PIO_num, PIO_DIR_OUT);
	pio_output(PIO_num, 0x01 );
}

/*********************************************************************************
void spi_set_cs()
	Set chip select pin LOW to start transfer.
inputs
	pio_cs: Chip select pin Number,
return value
	no
**********************************************************************************/
void SPI_set_cs( struct SPI_info *board_info )
{
	alt_u8 PIO_num = board_info->dev_info->pio_cs;
	pio_direction(PIO_num, PIO_DIR_OUT);
	pio_output(PIO_num, 0x00 );
}
