/*
 *	MAX10SoM-UART.c
 *
 *	Created on: 2015/04/30
 *		Author: Jinso(Nui)
 */

/* Include file */
#include "../inc/driver-common.h"
#include "../inc/MAX10SoM-UART.h"

/****************************************************************
void uart_init
	This function is called when initialize board.
	It is used to initialize UART device
inputs
	board_info  :information of the channel UART
	clock_source:frequency of the clock driving this component  ( in Hz)
return value
	void
*****************************************************************/
void uart_init(struct UART_info *board_info)
{
	alt_u16 divisor;
#ifdef  DRI_DEBUG
	printf(" Initializing  UART at 0x%x, \n\tClock source 0x%x \n\tBaud rate 0x%x \n\tInterrupt number 0x%x \n",board_info->address,board_info->clk,board_info->baud,board_info->irq.irq);
#endif
	divisor = (int)(board_info->clk/(board_info->baud) + 0.5);

	/*clear control register*/
	IOWR(board_info->address, UART_CTR, 0x0000);

	/*setting baud rate*/
	IOWR(board_info->address, UART_DIV, divisor);

	/* Fifo buffer initial */
	board_info->fifo.write = 0;
	board_info->fifo.read = 0;

	/* Interrupt register */
	if ( board_info->irq.isr != NULL ) {
		/*Enable interrupt for a read ready*/
		IOWR(board_info->address, UART_CTR, CTR_RRDY);

		/*interrupt handler*/
		irq_request( &board_info->irq );
	}
}

/****************************************************************
void uart_write
	assumes that any addressing and start
	has already been done.
	writes one byte of data to the base address
inputs
	board_info:information of the channel UART
	data	  :byte to write
return value
	void
*****************************************************************/
void uart_write(struct UART_info *board_info, alt_u8 data)
{
	alt_u32 base_add;
	base_add = board_info->address;

	/*waiting for data is transmitted ready*/
	while (( IORD(base_add, UART_STR) & STR_TRDY ) != STR_TRDY);

	/*write data to UART_TXR register*/
	IOWR(base_add, UART_TXR, data);
}

/****************************************************************
alt_u8 uart_read
	The function reads out the value of of the component
inputs
	board_info:information of the channel UART
	offset	  :offset to target register
return value
	the value of register base_add
*****************************************************************/
alt_u8 uart_read(struct UART_info *board_info)
{
	alt_u32 base_add;
	alt_u8 data;
	base_add = board_info->address;

	alt_u16 status;
	status = IORD(base_add, UART_STR);
	/*waiting for receive character ready*/
	while ((status & STR_RRDY) != STR_RRDY );
	/*reading data from UART_RXR register*/
	data = (char)IORD(base_add, UART_RXR);

	return data;
}

/****************************************************************
void uart0_irq_event
	This function is called when occur interrupt
inputs
	void
return value
	void
*****************************************************************/
void uart0_irq_event(void* context, alt_u32 id)
{
	handler_interrupt(&UART_board_info[0]);
}

/****************************************************************
void uart1_irq_event
	This function is called when occur interrupt
inputs
	void
return value
	void
*****************************************************************/
void uart1_irq_event(void* context, alt_u32 id)
{
	handler_interrupt(&UART_board_info[1]);
}

/****************************************************************
void handler_interrupt
	This function is called when handler interrupt UART0 and UART1
inputs
	board_info:information of the channel UART
return value
	void
*****************************************************************/
void handler_interrupt(struct UART_info *board_info)
{
	irq_disable(&(board_info->irq));

	/*receive data in fifo*/
	uart_rx_fifo(board_info);
	/* user function jump */
	board_info->irq.irq_event(NULL);

	irq_enable(&(board_info->irq));
}

/****************************************************************
void uart_rx_fifo(void)
	The function read data from UART_STR register
	and write that data to FIFO
inputs
	board_info:information of the channel UART
return value
	void
*****************************************************************/
void uart_rx_fifo(struct UART_info *board_info)
{
	alt_u8 data;
	alt_u16 wp;
	data =  uart_read(board_info);

	/*writing receive data to FIFO*/
	wp = board_info->fifo.write + 1;
	if(wp >= BUFFER_SIZE)
	{
		wp = 0;
	}

	if (wp != board_info->fifo.read)
	{
		board_info->fifo.buff[board_info->fifo.write] = data;
		board_info->fifo.write = wp;
	}
}
